using System;
using System.Collections;
using System.Collections.Generic;
using System.Xml;
using Server.Commands;
using Server.Commands.Generic;
using System.Text;
using System.Data;
using System.IO;
using Server;
using Server.Items;
using Server.Gumps;
using Server.Network;
using Server.Mobiles;

namespace Server.Misc
{
    public enum SkillGains { A_VerySlow, B_Slow, C_Normal, D_Fast, E_VeryFast, F_Automatic }

    public class ShardControl : Item
    {
        #region Variables and CommandProperties
        private static bool m_ProfanityProtection;
        private static ProfanityAction m_ProfanityAction;
        private static SkillGains m_Gains;
        private static bool m_ResFullHealth;
        private static bool m_PetResFullHealth;
        private static bool m_RegsEnabled;
        private static bool m_TCEnabled;
        private static bool m_AAC;
        private static int m_StartGold;
        private static int m_AccountsPerIP;
        private static TimeSpan m_Delay;
        private static TimeSpan m_Warning;
        private static int m_StatCap;
        private static int m_SkillsCap;
        private static TimeSpan m_SaveTime; //Expressed in Milliseconds as an Integer

        private static Dictionary<int, int> m_IntCap;
        private static Dictionary<int, int> m_DexCap;
        private static Dictionary<int, int> m_StrCap;

        private static int m_IndividualStatCap;

        public static Dictionary<int, int> IntCap { get { return m_IntCap; } set { m_IntCap = value; } }
        public static Dictionary<int, int> DexCap { get { return m_DexCap; } set { m_DexCap = value; } }
        public static Dictionary<int, int> StrCap { get { return m_StrCap; } set { m_StrCap = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static int IndividualStatCap { get { return m_IndividualStatCap; } set { m_IndividualStatCap = value; } }

        public static int LoadHalted = 0;

        [CommandProperty(AccessLevel.Administrator)]
        public static TimeSpan SaveTime { get { return m_SaveTime; } set { m_SaveTime = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static bool ProfanityProtection { get { return m_ProfanityProtection; } set { m_ProfanityProtection = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static ProfanityAction ProfanityAction { get { return m_ProfanityAction; } set { m_ProfanityAction = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static SkillGains Gains { get { return m_Gains; } set { m_Gains = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static bool ResFullHealth { get { return m_ResFullHealth; } set { m_ResFullHealth = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static bool PetResFullHealth { get { return m_PetResFullHealth; } set { m_PetResFullHealth = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static bool RegsEnabled { get { return m_RegsEnabled; } set { m_RegsEnabled = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static bool TCEnabled { get { return m_TCEnabled; } set { m_TCEnabled = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static bool AAC { get { return m_AAC; } set { m_AAC = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static int StartGold { get { return m_StartGold; } set { m_StartGold = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static int AccountsPerIP { get { return m_AccountsPerIP; } set { m_AccountsPerIP = value; } }

        [CommandProperty(AccessLevel.Administrator)]
        public static int StatCap { get { return m_StatCap; } set { m_StatCap = value; SetStats(value); } }

        [CommandProperty(AccessLevel.Administrator)]
        public static int SkillsCap { get { return m_SkillsCap; } set { m_SkillsCap = value; SetSkills(value); } }

        [CommandProperty(AccessLevel.Administrator)]
        public static TimeSpan Delay
        {
            get { return m_Delay; }
            set { m_Delay = value; }
        }

        [CommandProperty(AccessLevel.Administrator)]
        public static TimeSpan Warning
        {
            get { return m_Warning; }
            set { m_Warning = value; }
        }

        public override bool DisplayWeight { get { return ResetProperties(); } }
        #endregion

        #region Constructors and Initialization
        [Constructable]
        public ShardControl() : base(0x2DF4)
        {
            m_SaveTime = TimeSpan.Zero;
            LoadHalted = 0;
            try { LoadShardControls("default.xml", null); }
            catch
            {}
            if (LoadHalted > 0)
            {
                switch (LoadHalted)
                {
                    case 1: m_Delay = TimeSpan.FromMinutes(5.0); goto case 2;
                    case 2: m_Warning = TimeSpan.Zero; goto case 3;
                    case 3: m_TCEnabled = false; goto case 4;
                    case 4: m_StartGold = 1000; goto case 5;
                    case 5: m_AccountsPerIP = 1; goto case 6;
                    case 6: m_AAC = true; goto case 7;
                    case 7: m_StatCap = 225; goto case 8;
                    case 8: m_IndividualStatCap = 125; goto case 9;
                    case 9: m_SkillsCap = 7000; goto case 10;
                    case 10: m_RegsEnabled = true; goto case 11;
                    case 11: m_ResFullHealth = false; goto case 12;
                    case 12: m_PetResFullHealth = false; goto case 13;
                    case 13: m_ProfanityProtection = false; goto case 14;
                    case 14: m_ProfanityAction = ProfanityAction.None; goto case 15;
                    case 15: m_Gains = SkillGains.C_Normal; break;
                }
            }
            try { m_IntCap.Add(0, 0); m_IntCap.Remove(0); }
            catch { m_IntCap = new Dictionary<int, int>(); }
            try { m_DexCap.Add(0, 0); m_DexCap.Remove(0); }
            catch { m_DexCap = new Dictionary<int, int>(); }
            try { m_StrCap.Add(0, 0); m_StrCap.Remove(0); }
            catch { m_StrCap = new Dictionary<int, int>(); }
        }

        public ShardControl(Serial serial)
            : base(serial)
        {
        }

        public static int GetDexCap(Mobile m)
        {
            if (m_DexCap.ContainsKey(m.Serial.Value))
                return m_DexCap[m.Serial.Value];
            else
                return m_IndividualStatCap;
        }

        public static int GetStrCap(Mobile m)
        {
            if (m_StrCap.ContainsKey(m.Serial.Value))
                return m_StrCap[m.Serial.Value];
            else
                return m_IndividualStatCap;
        }

        public static int GetIntCap(Mobile m)
        {
            if (m_IntCap.ContainsKey(m.Serial.Value))
                return m_IntCap[m.Serial.Value];
            else
                return m_IndividualStatCap;
        }

        public static void Initialize()
        {
            CommandSystem.Register("ShardControl", AccessLevel.Administrator, new CommandEventHandler(ShardControl_OnCommand));
            CommandSystem.Register("Shard", AccessLevel.Administrator, new CommandEventHandler(ShardControl_OnCommand));
            CommandSystem.Register("StatCaps", AccessLevel.Player, new CommandEventHandler(StatCapsGump_OnCommand));
        }
        #endregion

        #region Methods
        public static void SetSkills(int newCap)
        {
            foreach (Mobile m in World.Mobiles.Values)
            {
                if (m is PlayerMobile) m.SkillsCap = newCap;
            }
        }

        public static void SetStats(int newCap)
        {
            foreach (Mobile m in World.Mobiles.Values)
            {
                if (m is PlayerMobile) m.StatCap = newCap;
            }
        }

        public override void OnDoubleClick(Mobile from)
        {
            from.SendGump(new ShardControlGump(from));
        }

        public bool ResetProperties()
        {
            InvalidateProperties();
            return false;
        }

        public override void GetProperties( ObjectPropertyList list )
        {
            list.Add (   "Shard Control");
            list.Add("------ by Lokai ------");
        }
        #endregion

        #region Serialization
        public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
            writer.Write((int)3); // version
            //version 3:
            writer.Write((int)m_IndividualStatCap);
            writer.Write((int)m_IntCap.Count);
            if (m_IntCap.Count > 0)
            {
                foreach (KeyValuePair<int, int> kvp in m_IntCap)
                {
                    writer.Write((int)kvp.Key);
                    writer.Write((int)kvp.Value);
                }
            }
            writer.Write((int)m_DexCap.Count);
            if (m_DexCap.Count > 0)
            {
                foreach (KeyValuePair<int, int> kvp in m_DexCap)
                {
                    writer.Write((int)kvp.Key);
                    writer.Write((int)kvp.Value);
                }
            }
            writer.Write((int)m_StrCap.Count);
            if (m_StrCap.Count > 0)
            {
                foreach (KeyValuePair<int, int> kvp in m_StrCap)
                {
                    writer.Write((int)kvp.Key);
                    writer.Write((int)kvp.Value);
                }
            }
            //version 2:
            writer.Write((TimeSpan)m_SaveTime);
            //version 1:
            writer.Write((bool)m_ProfanityProtection);
            writer.Write((int)m_ProfanityAction);
            writer.Write((int)m_Gains);
            writer.Write((int)m_StatCap);
            writer.Write((int)m_SkillsCap);
            writer.Write((bool)m_RegsEnabled);
            writer.Write((bool)m_ResFullHealth);
            writer.Write((bool)m_PetResFullHealth);
            //version 0:
            writer.Write((TimeSpan)m_Delay);
            writer.Write((TimeSpan)m_Warning);
            writer.Write((bool)m_TCEnabled);
            writer.Write((int)m_StartGold);
            writer.Write((int)m_AccountsPerIP);
            writer.Write((bool)m_AAC);
		}

		public override void Deserialize( GenericReader reader )
		{
            int count;
			base.Deserialize( reader );
			int version = reader.ReadInt();
            switch (version)
            {
                case 3:
                    {
                        m_IndividualStatCap = reader.ReadInt();
                        count = reader.ReadInt();
                        m_IntCap = new Dictionary<int, int>();
                        if (count > 0) for (int x = 0; x < count; x++)
                            {
                                m_IntCap.Add(reader.ReadInt(), reader.ReadInt());
                            }
                        count = reader.ReadInt();
                        m_DexCap = new Dictionary<int, int>();
                        if (count > 0) for (int x = 0; x < count; x++)
                            {
                                m_DexCap.Add(reader.ReadInt(), reader.ReadInt());
                            }
                        count = reader.ReadInt();
                        m_StrCap = new Dictionary<int, int>();
                        if (count > 0) for (int x = 0; x < count; x++)
                            {
                                m_StrCap.Add(reader.ReadInt(), reader.ReadInt());
                            }
                        goto case 2;
                    }
                case 2:
                    {
                        m_SaveTime = reader.ReadTimeSpan();
                        goto case 1;
                    }
                case 1:
                    {
                        m_ProfanityProtection = reader.ReadBool();
                        m_ProfanityAction = (ProfanityAction)reader.ReadInt();
                        m_Gains = (SkillGains)reader.ReadInt();
                        m_StatCap = reader.ReadInt();
                        m_SkillsCap = reader.ReadInt();
                        m_RegsEnabled = reader.ReadBool();
                        m_ResFullHealth = reader.ReadBool();
                        m_PetResFullHealth = reader.ReadBool();
                        goto case 0;
                    }
                case 0:
                    {
                        m_Delay = reader.ReadTimeSpan();
                        m_Warning = reader.ReadTimeSpan();
                        m_TCEnabled = reader.ReadBool();
                        m_StartGold = reader.ReadInt();
                        m_AccountsPerIP = reader.ReadInt();
                        m_AAC = reader.ReadBool();
                        break;
                    }
            }
        }
        #endregion

        #region ShardControl XML Processing
        private static void SaveShardControls(string filename, Mobile m)
        {
            string filePath = "Data/" + filename;
            bool autoSave = false;

            if (filePath == null || filePath.Length < 1) return;
            if (m == null || m.Deleted) autoSave = true;

            using (StreamWriter op = new StreamWriter(filePath))
            {
                if (op == null)
                {
                    m.SendMessage("unable to open file {0}", filePath);
                }
                XmlTextWriter xml = new XmlTextWriter(op);

                xml.Formatting = Formatting.Indented;
                xml.IndentChar = '\t';
                xml.Indentation = 1;

                xml.WriteStartDocument(true);

                xml.WriteStartElement("ShardControls");

                xml.WriteStartElement("Delay");
                xml.WriteString(ShardControl.Delay.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("Warning");
                xml.WriteString(ShardControl.Warning.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("TCEnabled");
                xml.WriteString(ShardControl.TCEnabled.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("StartGold");
                xml.WriteString(ShardControl.StartGold.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("AccountsPerIP");
                xml.WriteString(ShardControl.AccountsPerIP.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("AAC");
                xml.WriteString(ShardControl.AAC.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("StatCap");
                xml.WriteString(ShardControl.StatCap.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("IndividualStatCap");
                xml.WriteString(ShardControl.IndividualStatCap.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("SkillsCap");
                xml.WriteString(ShardControl.SkillsCap.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("RegsEnabled");
                xml.WriteString(ShardControl.RegsEnabled.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("ResFullHealth");
                xml.WriteString(ShardControl.ResFullHealth.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("PetResFullHealth");
                xml.WriteString(ShardControl.PetResFullHealth.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("ProfanityProtection");
                xml.WriteString(ShardControl.ProfanityProtection.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("ProfanityAction");
                xml.WriteString(ShardControl.ProfanityAction.ToString());
                xml.WriteEndElement();
                xml.WriteStartElement("Gains");
                xml.WriteString(ShardControl.Gains.ToString());
                xml.WriteEndElement();
                xml.WriteEndElement();

                xml.Close();
            }
            if (autoSave) Console.WriteLine("Settings saved to {0}", filePath);
            else m.SendMessage("Settings saved to {0}", filePath);
        }

        public static void LoadShardControls(string filename, Mobile m)
        {
            string filePath = "Data/" + filename;
            bool autoLoad = false;
            if (m == null || m.Deleted) autoLoad = true;
            if (filePath != null && filePath.Length >= 1)
            {

                if (File.Exists(filePath))
                {
                    XmlDocument doc = new XmlDocument();
                    doc.Load(filePath);

                    XmlElement root = doc["ShardControls"];
                    LoadDefaults(root);
                    if (autoLoad) Console.WriteLine("Settings loaded successfully from {0}", filePath);
                    else m.SendMessage("Settings loaded successfully from {0}", filePath);
                }
                else
                {
                    if (autoLoad) Console.WriteLine("File {0} does not exist.", filePath);
                    else m.SendMessage("File {0} does not exist.", filePath);
                    LoadHalted = 1;
                }
            }
            else
                LoadHalted = 1;
        }

        private static void LoadDefaults(XmlElement node)
        {
            try { ShardControl.Delay = TimeSpan.Parse(node["Delay"].InnerText); }
            catch { LoadHalted = 1; return; }
            try { ShardControl.Warning = TimeSpan.Parse(node["Warning"].InnerText); }
            catch { LoadHalted = 2; return; }
            try { ShardControl.TCEnabled = bool.Parse(node["TCEnabled"].InnerText); }
            catch { LoadHalted = 3; return; }
            try { ShardControl.StartGold = int.Parse(node["StartGold"].InnerText); }
            catch { LoadHalted = 4; return; }
            try { ShardControl.AccountsPerIP = int.Parse(node["AccountsPerIP"].InnerText); }
            catch { LoadHalted = 5; return; }
            try { ShardControl.AAC = bool.Parse(node["AAC"].InnerText); }
            catch { LoadHalted = 6; return; }
            try { ShardControl.StatCap = int.Parse(node["StatCap"].InnerText); }
            catch { LoadHalted = 7; return; }
            try { ShardControl.IndividualStatCap = int.Parse(node["IndividualStatCap"].InnerText); }
            catch { LoadHalted = 8; return; }
            try { ShardControl.SkillsCap = int.Parse(node["SkillsCap"].InnerText); }
            catch { LoadHalted = 9; return; }
            try { ShardControl.RegsEnabled = bool.Parse(node["RegsEnabled"].InnerText); }
            catch { LoadHalted = 10; return; }
            try { ShardControl.ResFullHealth = bool.Parse(node["ResFullHealth"].InnerText); }
            catch { LoadHalted = 11; return; }
            try { ShardControl.PetResFullHealth = bool.Parse(node["PetResFullHealth"].InnerText); }
            catch { LoadHalted = 12; return; }
            try { ShardControl.ProfanityProtection = bool.Parse(node["ProfanityProtection"].InnerText); }
            catch { LoadHalted = 13; return; }
            try { ShardControl.ProfanityAction = (ProfanityAction)Enum.Parse(ProfanityAction.GetType(), node["ProfanityAction"].InnerText); }
            catch { LoadHalted = 14; return; }
            try { ShardControl.Gains = (SkillGains)Enum.Parse(Gains.GetType(), node["Gains"].InnerText); }
            catch { LoadHalted = 15; return; }
        }
        #endregion

        #region Command Events
        [Usage("StatCaps")]
        [Description("Displays the StatCaps Gump.")]
        public static void StatCapsGump_OnCommand(CommandEventArgs e)
        {
            Mobile m = e.Mobile;
            if (m.Player) m.SendGump(new StatCapsGump(m));
        }

        [Usage("ShardControl [ShardControlOption value]")]
        [Description("Displays or changes the settings of the Shard Control.")]
        public static void ShardControl_OnCommand(CommandEventArgs e)
        {
            Mobile m = e.Mobile;
            if (m == null || m.Deleted) return;
            if (e.Arguments.Length >= 1)
            {
                if (e.Arguments[0].ToLower() == "help")
                {
                    m.SendGump(new ShardControlGump(m));
                    return;
                }
                if (e.Arguments.Length == 2)
                {
                    if (e.Arguments[0].ToLower() == "save")
                    {
                        SaveShardControls(e.Arguments[1], m);
                    }
                    else if (e.Arguments[0].ToLower() == "load")
                    {
                        LoadShardControls(e.Arguments[1], m);
                    }
                    else if (e.Arguments[0].ToLower() == "delay")
                    {
                        try
                        {
                            ShardControl.Delay = TimeSpan.FromMinutes(Convert.ToDouble(e.Arguments[1]));
                            m.SendMessage("Delay between saves:  {0} Minutes.", ShardControl.Delay.Minutes);
                        }
                        catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                    }
                    else if (e.Arguments[0].ToLower() == "warning")
                    {
                        try
                        {
                            ShardControl.Warning = TimeSpan.FromMinutes(Convert.ToDouble(e.Arguments[1]));
                            m.SendMessage("Warning before saves:  {0} Seconds.", ShardControl.Warning.Seconds);
                        }
                        catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                    }
                    else if (e.Arguments[0].ToLower() == "tcenabled")
                    {
                        try
                        {
                            ShardControl.TCEnabled = Convert.ToBoolean(e.Arguments[1]);
                            m.SendMessage("Test Center Setting:  {0}", ShardControl.TCEnabled ? "ON" : "OFF");
                        }
                        catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                    }
                    else if (e.Arguments[0].ToLower() == "startgold")
                    {
                        try
                        {
                            ShardControl.StartGold = Convert.ToInt32(e.Arguments[1]);
                            m.SendMessage("Starting Gold:  {0}.", ShardControl.StartGold);
                        }
                        catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                    }
                    else if (e.Arguments[0].ToLower().IndexOf("accounts") >= 0)
                    {
                        try
                        {
                            ShardControl.AccountsPerIP = Convert.ToInt32(e.Arguments[1]);
                            m.SendMessage("Accounts per IP address:  {0}.", ShardControl.AccountsPerIP);
                        }
                        catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                    }
                    else if (e.Arguments[0].ToLower() == "aac")
                    {
                        try
                        {
                            ShardControl.AAC = Convert.ToBoolean(e.Arguments[1]);
                            m.SendMessage("Automatic Account Creation:  {0}.", ShardControl.AAC ? "ON" : "OFF");
                        }
                        catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                    }
                    else
                        if (e.Arguments[0].ToLower().IndexOf("indiv") >= 0)
                        {
                            try
                            {
                                ShardControl.IndividualStatCap = Convert.ToInt32(e.Arguments[1]);
                                m.SendMessage("Individual Stat Cap:  {0}.", ShardControl.IndividualStatCap);
                            }
                            catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                        }
                        else
                            if (e.Arguments[0].ToLower() == "statcap")
                            {
                                try
                                {
                                    ShardControl.StatCap = Convert.ToInt32(e.Arguments[1]);
                                    m.SendMessage("Stat Cap:  {0}.", ShardControl.StatCap);
                                }
                                catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                            }
                            else
                                if (e.Arguments[0].ToLower() == "skillscap")
                                {
                                    try
                                    {
                                        ShardControl.SkillsCap = Convert.ToInt32(e.Arguments[1]);
                                        m.SendMessage("Skills Cap = {0}", (double)((double)ShardControl.SkillsCap) / 10);
                                    }
                                    catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                                }
                                else
                                    if (e.Arguments[0].ToLower() == "regsenabled")
                                    {
                                        try
                                        {
                                            ShardControl.RegsEnabled = Convert.ToBoolean(e.Arguments[1]);
                                            m.SendMessage("Reagents Use:  {0}.", ShardControl.RegsEnabled ? "ON" : "OFF");
                                        }
                                        catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                                    }
                                    else
                                        if (e.Arguments[0].ToLower() == "resfullhealth")
                                        {
                                            try
                                            {
                                                ShardControl.ResFullHealth = Convert.ToBoolean(e.Arguments[1]);
                                                m.SendMessage("Resurrect at full health:  {0}.", ShardControl.ResFullHealth ? "ON" : "OFF");
                                            }
                                            catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                                        }
                                        else
                                            if (e.Arguments[0].ToLower() == "petresfullhealth")
                                            {
                                                try
                                                {
                                                    ShardControl.PetResFullHealth = Convert.ToBoolean(e.Arguments[1]);
                                                    m.SendMessage("Pets res at full health:  {0}.", ShardControl.PetResFullHealth ? "ON" : "OFF");
                                                }
                                                catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                                            }
                                            else
                                                if (e.Arguments[0].ToLower().IndexOf("protection") >= 0)
                                                {
                                                    try
                                                    {
                                                        ShardControl.ProfanityProtection = Convert.ToBoolean(e.Arguments[1]);
                                                        m.SendMessage("Profanity Protection:  {0}.", ShardControl.ProfanityProtection ? "ON" : "OFF");
                                                    }
                                                    catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                                                }
                                                else
                                                    if (e.Arguments[0].ToLower().IndexOf("action") >= 0)
                                                    {
                                                        try
                                                        {
                                                            ShardControl.ProfanityAction = (ProfanityAction)Convert.ToInt32(e.Arguments[1]);
                                                            m.SendMessage("Profanity Action:  {0}.", ShardControl.ProfanityAction.ToString());
                                                        }
                                                        catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                                                    }
                                                    else
                                                        if (e.Arguments[0].ToLower() == "gains")
                                                        {
                                                            try
                                                            {
                                                                ShardControl.Gains = (SkillGains)Convert.ToInt32(e.Arguments[1]);
                                                                m.SendMessage("Skill Gains:  {0}.", ShardControl.Gains.ToString());
                                                            }
                                                            catch { m.SendMessage("invalid value : {0}", e.Arguments[1]); }
                                                        }
                                                        else
                                                        {
                                                            m.SendMessage("{0} : no such Shard Control option.", e.Arguments[0]);
                                                        }
                }

            }
            else // Display Shard Control Settings
            {
                m.SendMessage("Last Save took . . {0}.\n", ShardControl.SaveTime.ToString());
                m.SendMessage("Delay between saves: . . {0} Min.\n", ShardControl.Delay.Minutes);
                m.SendMessage("Warning before saves:. . {0} Sec.\n", ShardControl.Warning.Seconds);
                m.SendMessage("Test Center Setting: . . . {0}.\n", ShardControl.TCEnabled ? "ON" : "OFF");
                m.SendMessage("Starting Gold: . . . . . . . {0}.\n", ShardControl.StartGold);
                m.SendMessage("Accounts per IP address: . . {0}.\n", ShardControl.AccountsPerIP);
                m.SendMessage("Auto Account Creation:. . {0}.\n", ShardControl.AAC ? "ON" : "OFF");
                m.SendMessage("Individual Stat Cap: . . . {0}.\n", ShardControl.IndividualStatCap);
                m.SendMessage("Stat Cap:. . . . . . . . . {0}.\n", ShardControl.StatCap);
                m.SendMessage("Skills Cap:. . . . . . . . . {0}.\n", (double)((double)ShardControl.SkillsCap) / 10);
                m.SendMessage("Reagents Use:. . . . . . . {0}.\n", ShardControl.RegsEnabled ? "ON" : "OFF");
                m.SendMessage("Resurrect at full health: . {0}.\n", ShardControl.ResFullHealth ? "ON" : "OFF");
                m.SendMessage("Pets res at full health: . . {0}.\n", ShardControl.PetResFullHealth ? "ON" : "OFF");
                m.SendMessage("Profanity Protection: . . . {0}.\n", ShardControl.ProfanityProtection ? "ON" : "OFF");
                m.SendMessage("Profanity Action: . . . . {0}.\n", ShardControl.ProfanityAction.ToString());
                m.SendMessage("Skill Gains:. . . . . . {0}.", ShardControl.Gains.ToString());
            }
        }
        #endregion

    }

    public class ShardControlGump : Gump
    {
        public ShardControlGump( Mobile from )
            : base(20, 20)
        {
            bool Admin = from.AccessLevel >= AccessLevel.Administrator;
            StringBuilder list = new StringBuilder("---------------------------------------------\n");
            list.Append(" -------------- ShardControl  -----------------\n");
            list.Append("  --------------             -----------------\n");
            list.Append(" --------------   by Lokai   -----------------\n");
            list.Append("---------------------------------------------\n");
            list.Append(String.Format("Last Save took . . {0}.\n\n", ShardControl.SaveTime.ToString()));
            list.Append(String.Format("Delay between saves: . . {0} Min.\n", ShardControl.Delay.Minutes));
            list.Append(String.Format("Warning before saves:. . {0} Sec.\n", ShardControl.Warning.Seconds));
            list.Append(String.Format("Test Center Setting: . . . {0}.\n", ShardControl.TCEnabled ? "ON" : "OFF"));
            list.Append(String.Format("Starting Gold: . . . . . . . {0}.\n", ShardControl.StartGold));
            list.Append(String.Format("Accounts per IP address: . . {0}.\n", ShardControl.AccountsPerIP));
            list.Append(String.Format("Auto Account Creation:. . {0}.\n", ShardControl.AAC ? "ON" : "OFF"));
            list.Append(String.Format("Stat Cap:. . . . . . . . . {0}.\n", ShardControl.StatCap));
            list.Append(String.Format("Individual Stat Cap: . . . {0}.\n", ShardControl.IndividualStatCap));
            list.Append(String.Format("Skills Cap:. . . . . . . . . {0}.\n", (double)((double)ShardControl.SkillsCap) / 10));
            list.Append(String.Format("Reagents Use:. . . . . . . {0}.\n", ShardControl.RegsEnabled ? "ON" : "OFF"));
            list.Append(String.Format("Resurrect at full health: . {0}.\n", ShardControl.ResFullHealth ? "ON" : "OFF"));
            list.Append(String.Format("Pets res at full health: . . {0}.\n", ShardControl.PetResFullHealth ? "ON" : "OFF"));
            list.Append(String.Format("Profanity Protection: . . . {0}.\n", ShardControl.ProfanityProtection ? "ON" : "OFF"));
            list.Append(String.Format("Profanity Action: . . . . {0}.\n", ShardControl.ProfanityAction.ToString()));
            list.Append(String.Format("Skill Gains:. . . . . . {0}.", ShardControl.Gains.ToString()));

            AddPage(0);
            AddBackground(20, 20, 420, 550, 9300);
            AddHtml(30, 30, 400, 430, list.ToString(), true, false);

        }

        public override void OnResponse(NetState sender, RelayInfo info)
        {
        }
    }

    public class StatCapsGump : Gump
    {
        public StatCapsGump(Mobile from)
            : base(20, 20)
        {
            AddPage(0);
            AddBackground(20, 20, 100, 120, 9200);
            AddLabel(30, 30, 0, string.Format("Str Cap: {0}", ShardControl.GetStrCap(from)));
            AddLabel(30, 60, 0, string.Format("Dex Cap: {0}", ShardControl.GetDexCap(from)));
            AddLabel(30, 90, 0, string.Format("Int Cap: {0}", ShardControl.GetIntCap(from)));
        }

        public override void OnResponse(NetState sender, RelayInfo info)
        {
        }
    }
}
